// Fichier Game.java
/**
 * Représente la logique métier du jeu 2048. Gère l'état du plateau de jeu,
 * les déplacements, les fusions, le score de la partie en cours, et les conditions
 * de victoire ou de défaite. Cette classe est conçue pour être indépendante du
 * framework Android (pas de dépendance au Contexte ou aux SharedPreferences).
 */
package legion.muyue.best2048;

import androidx.annotation.NonNull;
import androidx.annotation.VisibleForTesting; // Pour les méthodes de test éventuelles

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class Game {

    /** Le plateau de jeu, une matrice 2D d'entiers. 0 représente une case vide. */
    private int[][] board;
    /** Générateur de nombres aléatoires pour l'ajout de nouvelles tuiles. */
    private final Random randomNumberGenerator;
    /** Score de la partie actuellement en cours. */
    private int currentScore = 0;
    /** Meilleur score global (reçu et stocké, mais non géré logiquement ici). */
    private int highestScore = 0;
    /** Taille du plateau de jeu (nombre de lignes/colonnes). */
    private static final int BOARD_SIZE = 4;
    /** Indicateur si la condition de victoire (>= 2048) a été atteinte. */
    private boolean gameWon = false;
    /** Indicateur si la partie est terminée (plus de mouvements possibles). */
    private boolean gameOver = false;

    /**
     * Constructeur pour démarrer une nouvelle partie.
     * Initialise un plateau vide, le score à 0, et ajoute deux tuiles initiales.
     */
    public Game() {
        this.randomNumberGenerator = new Random();
        initializeNewBoard();
    }

    /**
     * Constructeur pour restaurer une partie à partir d'un état sauvegardé.
     * Le meilleur score (`highestScore`) doit être défini séparément via {@link #setHighestScore(int)}.
     * Recalcule les états `gameWon` et `gameOver` en fonction du plateau fourni.
     *
     * @param board Le plateau de jeu restauré.
     * @param score Le score courant restauré.
     */
    public Game(int[][] board, int score) {
        // Valider les dimensions du plateau fourni ? Pourrait être ajouté.
        this.board = board;
        this.currentScore = score;
        this.randomNumberGenerator = new Random();
        checkWinCondition();
        checkGameOverCondition();
    }

    // --- Getters / Setters ---

    /**
     * Retourne la valeur de la tuile aux coordonnées spécifiées.
     * @param row Ligne (0 à BOARD_SIZE-1).
     * @param column Colonne (0 à BOARD_SIZE-1).
     * @return Valeur de la tuile, ou 0 si les coordonnées sont invalides.
     */
    public int getCellValue(int row, int column) {
        if (isIndexValid(row, column)) {
            return this.board[row][column];
        }
        return 0; // Retourne 0 pour indice invalide
    }

    /**
     * Définit la valeur d'une tuile aux coordonnées spécifiées.
     * Ne fait rien si les coordonnées sont invalides.
     * @param row Ligne (0 à BOARD_SIZE-1).
     * @param col Colonne (0 à BOARD_SIZE-1).
     * @param value Nouvelle valeur de la tuile.
     */
    public void setCellValue(int row, int col, int value) {
        if (isIndexValid(row, col)) {
            this.board[row][col] = value;
        }
    }

    /** @return Le score actuel de la partie. */
    public int getCurrentScore() { return currentScore; }

    /** @return Le meilleur score connu par cet objet (défini via setHighestScore). */
    public int getHighestScore() { return highestScore; }

    /**
     * Met à jour la valeur du meilleur score stockée dans cet objet Game.
     * Typiquement appelé par la classe gérant la persistance (MainActivity).
     * @param highScore Le meilleur score global à stocker.
     */
    public void setHighestScore(int highScore) { this.highestScore = highScore; }

    /** @return true si une tuile 2048 (ou plus) a été atteinte, false sinon. */
    public boolean isGameWon() { return gameWon; }

    /** @return true si aucune case n'est vide ET aucun mouvement/fusion n'est possible, false sinon. */
    public boolean isGameOver() { return gameOver; }

    /** Met à jour la valeur de gameWon si partie gagné **/
    private void setGameWon(boolean won) {this.gameWon = won;}

    /** Met à jour la valeur de gameWon si partie gagné **/
    private void setGameOver(boolean over) {this.gameOver = over;}

    /**
     * Retourne une copie profonde du plateau de jeu actuel.
     * Utile pour la sérialisation ou pour éviter des modifications externes non désirées.
     * @return Une nouvelle matrice 2D représentant l'état actuel du plateau.
     */
    public int[][] getBoard() {
        int[][] copy = new int[BOARD_SIZE][BOARD_SIZE];
        for(int i=0; i<BOARD_SIZE; i++) {
            System.arraycopy(this.board[i], 0, copy[i], 0, BOARD_SIZE);
        }
        return copy;
    }

    /**
     * (Ré)Initialise le plateau de jeu pour une nouvelle partie.
     * Remplit le plateau de zéros, réinitialise le score et les états `gameWon`/`gameOver`,
     * puis ajoute deux tuiles initiales.
     */
    private void initializeNewBoard() {
        this.board = new int[BOARD_SIZE][BOARD_SIZE]; // Crée une nouvelle matrice vide
        this.currentScore = 0;
        this.gameWon = false;
        this.gameOver = false;
        addNewTile(); // Ajoute la première tuile
        addNewTile(); // Ajoute la seconde tuile
    }

    // --- Logique du Jeu ---

    /**
     * Ajoute une nouvelle tuile (2, 4, 8, etc., selon probabilités) sur une case vide aléatoire.
     * Si le plateau est plein, cette méthode ne fait rien.
     */
    public void addNewTile() {
        List<int[]> emptyCells = findEmptyCells();
        if (!emptyCells.isEmpty()) {
            int[] randomCell = emptyCells.get(randomNumberGenerator.nextInt(emptyCells.size()));
            int value = generateRandomTileValue();
            setCellValue(randomCell[0], randomCell[1], value);
        }
    }

    /**
     * Trouve toutes les cellules vides sur le plateau.
     * @return Une liste de tableaux d'entiers `[row, col]` pour chaque cellule vide.
     */
    private List<int[]> findEmptyCells() {
        List<int[]> emptyCells = new ArrayList<>();
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                if (board[row][col] == 0) {
                    emptyCells.add(new int[]{row, col});
                }
            }
        }
        return emptyCells;
    }

    /**
     * Génère la valeur pour une nouvelle tuile en utilisant des probabilités prédéfinies.
     * @return La valeur (2, 4, 8, ...).
     */
    private int generateRandomTileValue() {
        int randomValue = randomNumberGenerator.nextInt(10000); // Base 10000 pour pourcentages fins
        if (randomValue < 8540) return 2;  // 85.40%
        if (randomValue < 9740) return 4;  // 12.00%
        if (randomValue < 9940) return 8;  //  2.00%
        if (randomValue < 9990) return 16; //  0.50%
        if (randomValue < 9995) return 32; //  0.05%
        if (randomValue < 9998) return 64; //  0.03%
        if (randomValue < 9999) return 128;//  0.01%
        return 256;                        //  0.01%
    }

    /**
     * Tente de déplacer et fusionner les tuiles vers le HAUT.
     * Met à jour le score interne et vérifie les états win/gameOver.
     * @return true si le plateau a été modifié, false sinon.
     */
    public boolean pushUp() { return processMove(MoveDirection.UP); }

    /**
     * Tente de déplacer et fusionner les tuiles vers le BAS.
     * @return true si le plateau a été modifié, false sinon.
     */
    public boolean pushDown() { return processMove(MoveDirection.DOWN); }

    /**
     * Tente de déplacer et fusionner les tuiles vers la GAUCHE.
     * @return true si le plateau a été modifié, false sinon.
     */
    public boolean pushLeft() { return processMove(MoveDirection.LEFT); }

    /**
     * Tente de déplacer et fusionner les tuiles vers la DROITE.
     * @return true si le plateau a été modifié, false sinon.
     */
    public boolean pushRight() { return processMove(MoveDirection.RIGHT); }

    /** Énumération interne pour clarifier le traitement des mouvements. */
    private enum MoveDirection { UP, DOWN, LEFT, RIGHT }

    /**
     * Méthode générique pour traiter un mouvement (déplacement et fusion) dans une direction donnée.
     * Contient la logique de base partagée par les méthodes pushX.
     * @param direction La direction du mouvement.
     * @return true si le plateau a été modifié, false sinon.
     */
    private boolean processMove(MoveDirection direction) {
        boolean boardChanged = false;
        // Itère sur l'axe perpendiculaire au mouvement
        for (int i = 0; i < BOARD_SIZE; i++) {
            boolean[] hasMerged = new boolean[BOARD_SIZE]; // Pour éviter double fusion sur l'axe de mouvement
            // Itère sur l'axe du mouvement, dans le bon sens
            int start = (direction == MoveDirection.DOWN || direction == MoveDirection.RIGHT) ? BOARD_SIZE - 2 : 1;
            int end = (direction == MoveDirection.DOWN || direction == MoveDirection.RIGHT) ? -1 : BOARD_SIZE;
            int step = (direction == MoveDirection.DOWN || direction == MoveDirection.RIGHT) ? -1 : 1;

            for (int j = start; j != end; j += step) {
                int row = (direction == MoveDirection.UP || direction == MoveDirection.DOWN) ? j : i;
                int col = (direction == MoveDirection.LEFT || direction == MoveDirection.RIGHT) ? j : i;

                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col);
                    int currentRow = row;
                    int currentCol = col;

                    // Calcule la position cible après déplacement dans les cases vides
                    int targetRow = currentRow;
                    int targetCol = currentCol;
                    int nextRow = targetRow + ((direction == MoveDirection.UP) ? -1 : (direction == MoveDirection.DOWN) ? 1 : 0);
                    int nextCol = targetCol + ((direction == MoveDirection.LEFT) ? -1 : (direction == MoveDirection.RIGHT) ? 1 : 0);

                    while (isIndexValid(nextRow, nextCol) && getCellValue(nextRow, nextCol) == 0) {
                        targetRow = nextRow;
                        targetCol = nextCol;
                        nextRow = targetRow + ((direction == MoveDirection.UP) ? -1 : (direction == MoveDirection.DOWN) ? 1 : 0);
                        nextCol = targetCol + ((direction == MoveDirection.LEFT) ? -1 : (direction == MoveDirection.RIGHT) ? 1 : 0);
                    }

                    // Déplace la tuile si sa position cible est différente
                    if (targetRow != currentRow || targetCol != currentCol) {
                        setCellValue(targetRow, targetCol, currentValue);
                        setCellValue(currentRow, currentCol, 0);
                        boardChanged = true;
                    }

                    // Vérifie la fusion potentielle avec la case suivante dans la direction du mouvement
                    int mergeTargetRow = targetRow + ((direction == MoveDirection.UP) ? -1 : (direction == MoveDirection.DOWN) ? 1 : 0);
                    int mergeTargetCol = targetCol + ((direction == MoveDirection.LEFT) ? -1 : (direction == MoveDirection.RIGHT) ? 1 : 0);
                    int mergeIndex = (direction == MoveDirection.UP || direction == MoveDirection.DOWN) ? mergeTargetRow : mergeTargetCol;

                    if (isIndexValid(mergeTargetRow, mergeTargetCol) &&
                            getCellValue(mergeTargetRow, mergeTargetCol) == currentValue &&
                            !hasMerged[mergeIndex])
                    {
                        int newValue = currentValue * 2;
                        setCellValue(mergeTargetRow, mergeTargetCol, newValue);
                        setCellValue(targetRow, targetCol, 0); // La tuile qui fusionne disparaît
                        currentScore += newValue;
                        hasMerged[mergeIndex] = true;
                        boardChanged = true;
                    }
                }
            }
        }
        // Vérifie les conditions de fin après chaque type de mouvement complet
        checkWinCondition();
        checkGameOverCondition();
        return boardChanged;
    }

    /**
     * Vérifie si les indices de ligne et colonne sont valides pour le plateau.
     * @param row Ligne.
     * @param col Colonne.
     * @return true si les indices sont dans les limites [0, BOARD_SIZE-1].
     */
    private boolean isIndexValid(int row, int col) {
        return row >= 0 && row < BOARD_SIZE && col >= 0 && col < BOARD_SIZE;
    }


    /**
     * Sérialise l'état essentiel du jeu (plateau et score courant) pour sauvegarde.
     * Format: "val,val,...,val,score"
     * @return Chaîne représentant l'état du jeu.
     */
    @NonNull
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) { sb.append(board[row][col]).append(","); }
        }
        sb.append(currentScore);
        return sb.toString();
    }

    /**
     * Crée un objet Game à partir de sa représentation sérialisée (plateau + score).
     * Le meilleur score doit être défini séparément après la création.
     * @param serializedState La chaîne issue de {@link #toString()}.
     * @return Une nouvelle instance de Game, ou null en cas d'erreur de format.
     */
    public static Game deserialize(String serializedState) {
        if (serializedState == null || serializedState.isEmpty()) return null;
        String[] values = serializedState.split(",");
        if (values.length != (BOARD_SIZE * BOARD_SIZE + 1)) return null; // +1 pour le score
        int[][] newBoard = new int[BOARD_SIZE][BOARD_SIZE]; int index = 0;
        try {
            for (int row = 0; row < BOARD_SIZE; row++) {
                for (int col = 0; col < BOARD_SIZE; col++) { newBoard[row][col] = Integer.parseInt(values[index++]); }
            }
            int score = Integer.parseInt(values[index]); // Le dernier élément est le score
            return new Game(newBoard, score);
        } catch (NumberFormatException | ArrayIndexOutOfBoundsException e) { return null; }
    }

    /**
     * Vérifie si la condition de victoire (une tuile >= 2048) est atteinte.
     * Met à jour l'état interne `gameWon`.
     */
    private void checkWinCondition() {
        if (!gameWon) {
            for (int r=0; r<BOARD_SIZE; r++) for (int c=0; c<BOARD_SIZE; c++) if (getCellValue(r,c) >= 2048) {
                setGameWon(true); return;
            }
        }
    }

    /**
     * Vérifie si la condition de fin de partie est atteinte (plateau plein ET aucun mouvement possible).
     * Met à jour l'état interne `gameOver`.
     */
    private void checkGameOverCondition() {
        if (hasEmptyCell()) { setGameOver(false); return; } // Pas game over si case vide
        // Vérifie s'il existe au moins une fusion possible
        for (int r=0; r<BOARD_SIZE; r++) for (int c=0; c<BOARD_SIZE; c++) {
            int current = getCellValue(r,c);
            if ((r>0 && getCellValue(r-1,c)==current) || (r<BOARD_SIZE-1 && getCellValue(r+1,c)==current) ||
                    (c>0 && getCellValue(r,c-1)==current) || (c<BOARD_SIZE-1 && getCellValue(r,c+1)==current)) {
                setGameOver(false); return; // Fusion possible -> pas game over
            }
        }
        setGameOver(true); // Aucune case vide et aucune fusion -> game over
    }

    /**
     * Vérifie si le plateau contient au moins une case vide (valeur 0).
     * @return true si une case vide existe, false sinon.
     */
    private boolean hasEmptyCell() {
        for (int r=0; r<BOARD_SIZE; r++) for (int c=0; c<BOARD_SIZE; c++) if (getCellValue(r,c)==0) return true;
        return false;
    }

    /**
     * Retourne la valeur de la plus haute tuile présente sur le plateau.
     * @return La valeur maximale trouvée, ou 0 si le plateau est vide.
     */
    public int getHighestTileValue() {
        int maxTile = 0;
        for (int r=0; r<BOARD_SIZE; r++) for (int c=0; c<BOARD_SIZE; c++) if (board[r][c] > maxTile) maxTile = board[r][c];
        return maxTile;
    }
}