// Fichier MainActivity.java
/**
 * Activité principale de l'application 2048.
 * Gère l'interface utilisateur (plateau, scores, boutons), coordonne les interactions
 * avec la logique du jeu (classe Game) et la gestion des statistiques (classe GameStats).
 * Gère également le cycle de vie de l'application et la persistance de l'état du jeu.
 */
package legion.muyue.best2048;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewStub;
import android.view.animation.AnimationUtils;
import android.widget.TextView;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.gridlayout.widget.GridLayout;
import android.widget.Button;


public class MainActivity extends AppCompatActivity {


    private GridLayout boardGridLayout;
    private TextView currentScoreTextView;
    private TextView highestScoreTextView;
    private Button newGameButton;
    private Button multiplayerButton;
    private Button statisticsButton;
    private Button menuButton;
    private ViewStub statisticsViewStub;
    private View inflatedStatsView;


    private Game game;
    private GameStats gameStats;
    private static final int BOARD_SIZE = 4;


    private boolean statisticsVisible = false;
    private enum GameFlowState { PLAYING, WON_DIALOG_SHOWN, GAME_OVER }
    private GameFlowState currentGameState = GameFlowState.PLAYING;


    private SharedPreferences preferences;
    private static final String PREFS_NAME = "Best2048_Prefs";
    private static final String HIGH_SCORE_KEY = "high_score";
    private static final String GAME_STATE_KEY = "game_state";



    @Override
    protected void onCreate(Bundle savedInstanceState) {
        EdgeToEdge.enable(this);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        findViews();
        initializeGameAndStats();
        setupListeners();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (game != null && gameStats != null && !game.isGameOver() && !game.isGameWon()) {
            gameStats.setCurrentGameStartTimeMs(System.currentTimeMillis());
        }

        if (statisticsVisible) {
            if (inflatedStatsView != null) {
                updateStatisticsTextViews();
                inflatedStatsView.setVisibility(View.VISIBLE);
                multiplayerButton.setVisibility(View.GONE);
            } else {

                toggleStatistics();
            }
        }
    }

    @Override
    protected void onPause() {
        super.onPause();

        if (game != null && gameStats != null) {

            if (!game.isGameOver() && !game.isGameWon()) {
                gameStats.addPlayTime(System.currentTimeMillis() - gameStats.getCurrentGameStartTimeMs());
            }
            saveGame();
            gameStats.saveStats();
        }
    }



    /**
     * Récupère les références des vues du layout principal via leur ID.
     */
    private void findViews() {
        boardGridLayout = findViewById(R.id.gameBoard);
        currentScoreTextView = findViewById(R.id.scoreLabel);
        highestScoreTextView = findViewById(R.id.highScoreLabel);
        newGameButton = findViewById(R.id.restartButton);
        statisticsButton = findViewById(R.id.statsButton);
        menuButton = findViewById(R.id.menuButton);
        multiplayerButton = findViewById(R.id.multiplayerButton);
        statisticsViewStub = findViewById(R.id.statsViewStub);
    }

    /**
     * Initialise les objets Game et GameStats.
     * Charge l'état du jeu sauvegardé (s'il existe) et le meilleur score.
     * Met à jour l'interface utilisateur initiale.
     */
    private void initializeGameAndStats() {
        preferences = getSharedPreferences(PREFS_NAME, MODE_PRIVATE);
        gameStats = new GameStats(this);
        loadGame();
        updateUI();
        if (game == null) {
            startNewGame();
        } else {

            if (game.isGameOver()) {
                currentGameState = GameFlowState.GAME_OVER;
            } else if (game.isGameWon()) {

                currentGameState = GameFlowState.WON_DIALOG_SHOWN;
            } else {
                currentGameState = GameFlowState.PLAYING;

            }
        }
    }

    /**
     * Attache les listeners aux boutons et configure le listener de swipe sur le plateau.
     */
    private void setupListeners() {
        newGameButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showRestartConfirmationDialog();
        });
        statisticsButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            toggleStatistics();
        });
        menuButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showMenu();
        });
        multiplayerButton.setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showMultiplayerScreen();
        });
        setupSwipeListener();
    }



    /**
     * Met à jour complètement l'interface utilisateur (plateau et scores).
     */
    private void updateUI() {
        if (game == null) return;
        updateBoard();
        updateScores();
    }

    /**
     * Redessine le plateau de jeu en créant/mettant à jour les TextViews des tuiles.
     */
    private void updateBoard() {
        boardGridLayout.removeAllViews();
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                TextView tileTextView = new TextView(this);
                int value = game.getCellValue(row, col);
                setTileStyle(tileTextView, value);

                GridLayout.LayoutParams params = new GridLayout.LayoutParams();
                params.width = 0; params.height = 0;
                params.rowSpec = GridLayout.spec(row, 1f);
                params.columnSpec = GridLayout.spec(col, 1f);
                int margin = (int) getResources().getDimension(R.dimen.tile_margin);
                params.setMargins(margin, margin, margin, margin);
                tileTextView.setLayoutParams(params);
                boardGridLayout.addView(tileTextView);
            }
        }
    }

    /**
     * Met à jour les TextViews affichant le score courant et le meilleur score.
     */
    private void updateScores() {
        currentScoreTextView.setText(getString(R.string.score_placeholder, game.getCurrentScore()));
        highestScoreTextView.setText(getString(R.string.high_score_placeholder, game.getHighestScore()));
    }

    /**
     * Applique le style visuel (fond, texte, taille) à une TextView représentant une tuile.
     * @param tileTextView La TextView de la tuile.
     * @param value La valeur numérique de la tuile (0 pour vide).
     */
    private void setTileStyle(TextView tileTextView, int value) {

        tileTextView.setText(value > 0 ? String.valueOf(value) : "");
        tileTextView.setGravity(Gravity.CENTER);
        tileTextView.setTypeface(null, android.graphics.Typeface.BOLD);
        int backgroundColorId; int textColorId; int textSizeId;
        switch (value) {
            case 0:    backgroundColorId = R.color.tile_empty; textColorId = android.R.color.transparent; textSizeId = R.dimen.text_size_tile_small; break;
            case 2:    backgroundColorId = R.color.tile_2;     textColorId = R.color.text_tile_low;     textSizeId = R.dimen.text_size_tile_small; break;
            case 4:    backgroundColorId = R.color.tile_4;     textColorId = R.color.text_tile_low;     textSizeId = R.dimen.text_size_tile_small; break;
            case 8:    backgroundColorId = R.color.tile_8;     textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_small; break;
            case 16:   backgroundColorId = R.color.tile_16;    textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_small; break;
            case 32:   backgroundColorId = R.color.tile_32;    textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_small; break;
            case 64:   backgroundColorId = R.color.tile_64;    textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_small; break;
            case 128:  backgroundColorId = R.color.tile_128;   textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_medium; break;
            case 256:  backgroundColorId = R.color.tile_256;   textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_medium; break;
            case 512:  backgroundColorId = R.color.tile_512;   textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_medium; break;
            case 1024: backgroundColorId = R.color.tile_1024;  textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_large; break;
            case 2048: backgroundColorId = R.color.tile_2048;  textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_large; break;
            default:   backgroundColorId = R.color.tile_super;   textColorId = R.color.text_tile_high;    textSizeId = R.dimen.text_size_tile_large; break;
        }
        tileTextView.setBackgroundResource(R.drawable.tile_background);
        tileTextView.getBackground().setTint(ContextCompat.getColor(this, backgroundColorId));
        tileTextView.setTextColor(ContextCompat.getColor(this, textColorId));
        tileTextView.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(textSizeId));
    }




    /**
     * Configure le listener pour détecter les swipes sur le plateau de jeu.
     */
    @SuppressLint("ClickableViewAccessibility")
    private void setupSwipeListener() {
        boardGridLayout.setOnTouchListener(new OnSwipeTouchListener(this, new OnSwipeTouchListener.SwipeListener() {
            @Override public void onSwipeTop() { handleSwipe(Direction.UP); }
            @Override public void onSwipeBottom() { handleSwipe(Direction.DOWN); }
            @Override public void onSwipeLeft() { handleSwipe(Direction.LEFT); }
            @Override public void onSwipeRight() { handleSwipe(Direction.RIGHT); }
        }));
    }

    /**
     * Traite un geste de swipe de l'utilisateur sur le plateau de jeu.
     * 1. Tente d'effectuer le mouvement dans l'objet Game.
     * 2. Si le mouvement a modifié le plateau (boardChanged == true) :
     * - Met à jour les statistiques (mouvements, fusions, score, etc.).
     * - Ajoute une nouvelle tuile aléatoire.
     * - Met à jour l'affichage (UI).
     * 3. Vérifie l'état du jeu (gagné ou perdu) APRÈS la tentative de mouvement,
     * que le plateau ait changé ou non. C'est la correction clé.
     * 4. Affiche les boîtes de dialogue appropriées (victoire, défaite).
     *
     * @param direction La direction du swipe détecté (UP, DOWN, LEFT, RIGHT).
     */
    private void handleSwipe(Direction direction) {

        if (game == null || gameStats == null || currentGameState == GameFlowState.GAME_OVER) {
            return;
        }


        int scoreBefore = game.getCurrentScore();

        boolean boardChanged = false;




        switch (direction) {
            case UP:
                boardChanged = game.pushUp();
                break;
            case DOWN:
                boardChanged = game.pushDown();
                break;
            case LEFT:
                boardChanged = game.pushLeft();
                break;
            case RIGHT:
                boardChanged = game.pushRight();
                break;
        }


        if (boardChanged) {

            gameStats.recordMove();
            int scoreAfter = game.getCurrentScore();
            int scoreDelta = scoreAfter - scoreBefore;
            if (scoreDelta > 0) {

                gameStats.recordMerge(1);

                if (scoreAfter > game.getHighestScore()) {
                    game.setHighestScore(scoreAfter);
                    gameStats.setHighestScore(scoreAfter);
                }
            }

            gameStats.updateHighestTile(game.getHighestTileValue());


            game.addNewTile();


            updateUI();

        }




        if (currentGameState != GameFlowState.GAME_OVER) {



            if (game.isGameWon() && currentGameState == GameFlowState.PLAYING) {
                currentGameState = GameFlowState.WON_DIALOG_SHOWN;

                long timeTaken = System.currentTimeMillis() - gameStats.getCurrentGameStartTimeMs();
                gameStats.recordWin(timeTaken);

                showGameWonKeepPlayingDialog();



            } else if (game.isGameOver()) {
                currentGameState = GameFlowState.GAME_OVER;

                long timeTaken = System.currentTimeMillis() - gameStats.getCurrentGameStartTimeMs();
                gameStats.recordLoss();
                gameStats.endGame(timeTaken);

                showGameOverDialog();

                if (!boardChanged) {
                    updateUI();
                }
            }

        }
    }

    enum Direction { UP, DOWN, LEFT, RIGHT }

    /**
     * Affiche la boîte de dialogue demandant confirmation avant de redémarrer.
     */
    private void showRestartConfirmationDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        LayoutInflater inflater = getLayoutInflater(); View dialogView = inflater.inflate(R.layout.dialog_restart_confirm, null);
        builder.setView(dialogView); Button cancelButton = dialogView.findViewById(R.id.dialogCancelButton); Button confirmButton = dialogView.findViewById(R.id.dialogConfirmButton);
        final AlertDialog dialog = builder.create(); cancelButton.setOnClickListener(v -> dialog.dismiss());
        confirmButton.setOnClickListener(v -> { dialog.dismiss(); startNewGame(); }); dialog.show();
    }

    /**
     * Démarre une nouvelle partie : initialise GameStats pour une nouvelle session,
     * crée un nouvel objet Game, synchronise le meilleur score et met à jour l'UI.
     */
    private void startNewGame() {
        gameStats.startGame();
        game = new Game();
        game.setHighestScore(gameStats.getOverallHighScore());
        currentGameState = GameFlowState.PLAYING;
        updateUI();
    }

    /**
     * Affiche la boîte de dialogue quand 2048 est atteint, en utilisant un layout personnalisé.
     * Propose de continuer à jouer ou de commencer une nouvelle partie.
     */
    private void showGameWonKeepPlayingDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        LayoutInflater inflater = getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.dialog_game_won, null);
        builder.setView(dialogView);
        builder.setCancelable(false);


        Button keepPlayingButton = dialogView.findViewById(R.id.dialogKeepPlayingButton);
        Button newGameButton = dialogView.findViewById(R.id.dialogNewGameButtonWon);

        final AlertDialog dialog = builder.create();

        keepPlayingButton.setOnClickListener(v -> {

            dialog.dismiss();
        });

        newGameButton.setOnClickListener(v -> {
            dialog.dismiss();
            startNewGame();
        });

        dialog.show();
    }

    /**
     * Affiche la boîte de dialogue de fin de partie (plus de mouvements), en utilisant un layout personnalisé.
     * Propose Nouvelle Partie ou Quitter.
     */
    private void showGameOverDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        LayoutInflater inflater = getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.dialog_game_over, null);
        builder.setView(dialogView);
        builder.setCancelable(false);


        TextView messageTextView = dialogView.findViewById(R.id.dialogMessageGameOver);
        Button newGameButton = dialogView.findViewById(R.id.dialogNewGameButtonGameOver);
        Button quitButton = dialogView.findViewById(R.id.dialogQuitButtonGameOver);


        messageTextView.setText(getString(R.string.game_over_message, game.getCurrentScore()));

        final AlertDialog dialog = builder.create();

        newGameButton.setOnClickListener(v -> {
            dialog.dismiss();
            startNewGame();
        });

        quitButton.setOnClickListener(v -> {
            dialog.dismiss();
            finish();
        });

        dialog.show();
    }




    /**
     * Affiche ou masque le panneau de statistiques.
     * Gonfle le layout via ViewStub si c'est la première fois.
     */
    private void toggleStatistics() {
        statisticsVisible = !statisticsVisible;
        if (statisticsVisible) {
            if (inflatedStatsView == null) {
                inflatedStatsView = statisticsViewStub.inflate();

                Button backButton = inflatedStatsView.findViewById(R.id.backButton);
                backButton.setOnClickListener(v -> toggleStatistics());
            }
            updateStatisticsTextViews();
            inflatedStatsView.setVisibility(View.VISIBLE);
            multiplayerButton.setVisibility(View.GONE);
        } else {
            if (inflatedStatsView != null) {
                inflatedStatsView.setVisibility(View.GONE);
            }
            multiplayerButton.setVisibility(View.VISIBLE);
        }
    }

    /**
     * Remplit les TextViews du panneau de statistiques avec les données de l'objet GameStats.
     * Doit être appelé seulement après que `inflatedStatsView` a été initialisé.
     */
    private void updateStatisticsTextViews() {
        if (inflatedStatsView == null || gameStats == null) return;


        TextView highScoreStatsLabel = inflatedStatsView.findViewById(R.id.high_score_stats_label);
        TextView totalGamesPlayedLabel = inflatedStatsView.findViewById(R.id.total_games_played_label);
        TextView totalGamesStartedLabel = inflatedStatsView.findViewById(R.id.total_games_started_label);
        TextView winPercentageLabel = inflatedStatsView.findViewById(R.id.win_percentage_label);
        TextView totalPlayTimeLabel = inflatedStatsView.findViewById(R.id.total_play_time_label);
        TextView totalMovesLabel = inflatedStatsView.findViewById(R.id.total_moves_label);
        TextView currentMovesLabel = inflatedStatsView.findViewById(R.id.current_moves_label);
        TextView currentGameTimeLabel = inflatedStatsView.findViewById(R.id.current_game_time_label);
        TextView averageGameTimeLabel = inflatedStatsView.findViewById(R.id.average_game_time_label);
        TextView bestWinningTimeLabel = inflatedStatsView.findViewById(R.id.best_winning_time_label);
        TextView worstWinningTimeLabel = inflatedStatsView.findViewById(R.id.worst_winning_time_label);
        TextView totalMergesLabel = inflatedStatsView.findViewById(R.id.total_merges_label);
        TextView highestTileLabel = inflatedStatsView.findViewById(R.id.highest_tile_label);
        TextView objectiveReachedLabel = inflatedStatsView.findViewById(R.id.number_of_time_objective_reached_label);
        TextView perfectGameLabel = inflatedStatsView.findViewById(R.id.perfect_game_label);
        TextView multiplayerGamesWonLabel = inflatedStatsView.findViewById(R.id.multiplayer_games_won_label);
        TextView multiplayerGamesPlayedLabel = inflatedStatsView.findViewById(R.id.multiplayer_games_played_label);
        TextView multiplayerWinRateLabel = inflatedStatsView.findViewById(R.id.multiplayer_win_rate_label);
        TextView multiplayerBestWinningStreakLabel = inflatedStatsView.findViewById(R.id.multiplayer_best_winning_streak_label);
        TextView multiplayerAverageScoreLabel = inflatedStatsView.findViewById(R.id.multiplayer_average_score_label);
        TextView averageTimePerGameMultiLabel = inflatedStatsView.findViewById(R.id.average_time_per_game_label);
        TextView totalMultiplayerLossesLabel = inflatedStatsView.findViewById(R.id.total_multiplayer_losses_label);
        TextView multiplayerHighScoreLabel = inflatedStatsView.findViewById(R.id.multiplayer_high_score_label);
        TextView mergesThisGameLabel = inflatedStatsView.findViewById(R.id.merges_this_game);


        highScoreStatsLabel.setText(getString(R.string.high_score_stats, gameStats.getOverallHighScore()));
        totalGamesPlayedLabel.setText(getString(R.string.total_games_played, gameStats.getTotalGamesPlayed()));
        totalGamesStartedLabel.setText(getString(R.string.total_games_started, gameStats.getTotalGamesStarted()));
        totalMovesLabel.setText(getString(R.string.total_moves, gameStats.getTotalMoves()));
        currentMovesLabel.setText(getString(R.string.current_moves, gameStats.getCurrentMoves()));
        mergesThisGameLabel.setText(getString(R.string.merges_this_game_label, gameStats.getMergesThisGame()));
        totalMergesLabel.setText(getString(R.string.total_merges, gameStats.getTotalMerges()));
        highestTileLabel.setText(getString(R.string.highest_tile, gameStats.getHighestTile()));
        objectiveReachedLabel.setText(getString(R.string.number_of_time_objective_reached, gameStats.getNumberOfTimesObjectiveReached()));
        perfectGameLabel.setText(getString(R.string.perfect_games, gameStats.getPerfectGames()));
        multiplayerGamesWonLabel.setText(getString(R.string.multiplayer_games_won, gameStats.getMultiplayerGamesWon()));
        multiplayerGamesPlayedLabel.setText(getString(R.string.multiplayer_games_played, gameStats.getMultiplayerGamesPlayed()));
        multiplayerBestWinningStreakLabel.setText(getString(R.string.multiplayer_best_winning_streak, gameStats.getMultiplayerBestWinningStreak()));
        multiplayerAverageScoreLabel.setText(getString(R.string.multiplayer_average_score, gameStats.getMultiplayerAverageScore()));
        totalMultiplayerLossesLabel.setText(getString(R.string.total_multiplayer_losses, gameStats.getTotalMultiplayerLosses()));
        multiplayerHighScoreLabel.setText(getString(R.string.multiplayer_high_score, gameStats.getMultiplayerHighestScore()));


        String winPercentage = (gameStats.getTotalGamesStarted() > 0) ? String.format("%.2f%%", ((double) gameStats.getNumberOfTimesObjectiveReached() / gameStats.getTotalGamesStarted()) * 100) : "N/A";
        winPercentageLabel.setText(getString(R.string.win_percentage, winPercentage));
        String multiplayerWinRate = (gameStats.getMultiplayerGamesPlayed() > 0) ? String.format("%.2f%%", ((double) gameStats.getMultiplayerGamesWon() / gameStats.getMultiplayerGamesPlayed()) * 100) : "N/A";
        multiplayerWinRateLabel.setText(getString(R.string.multiplayer_win_rate, multiplayerWinRate));


        totalPlayTimeLabel.setText(getString(R.string.total_play_time, GameStats.formatTime(gameStats.getTotalPlayTimeMs())));
        long currentDuration = (game != null && !game.isGameOver() && !game.isGameWon() && gameStats != null) ? System.currentTimeMillis() - gameStats.getCurrentGameStartTimeMs() : 0;
        currentGameTimeLabel.setText(getString(R.string.current_game_time, GameStats.formatTime(currentDuration)));
        averageGameTimeLabel.setText(getString(R.string.average_time_per_game, GameStats.formatTime(gameStats.getAverageGameTimeMs())));
        averageTimePerGameMultiLabel.setText(getString(R.string.average_time_per_game_label, GameStats.formatTime(gameStats.getMultiplayerAverageTimeMs())));
        bestWinningTimeLabel.setText(getString(R.string.best_winning_time, (gameStats.getBestWinningTimeMs() != Long.MAX_VALUE) ? GameStats.formatTime(gameStats.getBestWinningTimeMs()) : "N/A"));
        worstWinningTimeLabel.setText(getString(R.string.worst_winning_time, (gameStats.getWorstWinningTimeMs() != 0) ? GameStats.formatTime(gameStats.getWorstWinningTimeMs()) : "N/A"));
    }



    /** Affiche un dialogue placeholder pour le menu. */
    private void showMenu() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Menu").setMessage("Fonctionnalité de menu à venir !").setPositiveButton("OK", null);
        builder.create().show();
    }

    /** Affiche un dialogue placeholder pour le multijoueur. */
    private void showMultiplayerScreen() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Multijoueur").setMessage("Fonctionnalité multijoueur à venir !").setPositiveButton("OK", null);
        builder.create().show();
    }



    /** Sauvegarde l'état du jeu et le meilleur score via SharedPreferences. */
    private void saveGame() {
        SharedPreferences.Editor editor = preferences.edit();
        if (game != null) {
            editor.putString(GAME_STATE_KEY, game.toString());
            editor.putInt(HIGH_SCORE_KEY, game.getHighestScore());
        } else {
            editor.remove(GAME_STATE_KEY);
        }
        editor.apply();
    }

    /** Charge l'état du jeu depuis SharedPreferences et synchronise le meilleur score. */
    private void loadGame() {
        String gameStateString = preferences.getString(GAME_STATE_KEY, null);
        int savedHighScore = preferences.getInt(HIGH_SCORE_KEY, 0);

        if (gameStats != null) {
            gameStats.setHighestScore(savedHighScore);
        }

        if (gameStateString != null) {
            game = Game.deserialize(gameStateString);
            if (game != null) {
                game.setHighestScore(savedHighScore);

                if (game.isGameOver()) currentGameState = GameFlowState.GAME_OVER;
                else if (game.isGameWon()) currentGameState = GameFlowState.WON_DIALOG_SHOWN;
                else currentGameState = GameFlowState.PLAYING;
            } else { game = null; }
        } else { game = null; }

        if (game == null) {
            game = new Game();
            game.setHighestScore(savedHighScore);
            currentGameState = GameFlowState.PLAYING;
        }
    }

}